@@ -8,8 +8,9 @@
 //! gifski_set_file_output(g, "file.gif");
 //!
 //! for(int i=0; i < frames; i++) {
-//!      int res = gifski_add_frame_rgba(g, i, width, height, buffer, 5);
-//!      if (res != GIFSKI_OK) break;
+//!     // added frames will be sorted by the `i` index
+//!     int res = gifski_add_frame_rgba(g, i, width, height, buffer, 5);
+//!     if (res != GIFSKI_OK) break;
 //! }
 //! int res = gifski_finish(g);
 //! if (res != GIFSKI_OK) return;
@@ -42,11 +43,12 @@
 //!
 //! it will build `target/aarch64-apple-ios/release/libgifski.a` (ignore the warning about cdylib).
 
-use crate::{Collector, NoProgress, ProgressCallback, ProgressReporter, Repeat, Settings, Writer};
+use crate::progress::ProgressCallback;
+use crate::{CCallbacks, Collector, ErrorCallback, ProgressReporter, Repeat, Settings, Writer};
 use imgref::{Img, ImgVec};
 use rgb::{RGB8, RGBA8};
 use std::fs;
-use std::ffi::{CStr, CString};
+use std::ffi::CStr;
 use std::fs::File;
 use std::io;
 use std::io::Write;
@@ -56,7 +58,7 @@ use std::path::{Path, PathBuf};
 use std::ptr;
 use std::slice;
 use std::thread;
-use std::sync::{Arc, Mutex};
+use std::sync::{Arc, Mutex, MutexGuard};
 mod c_api_error;
 use self::c_api_error::GifskiError;
 use std::panic::catch_unwind;
@@ -95,8 +97,8 @@ pub struct GifskiHandle {
 pub struct GifskiHandleInternal {
     writer: Mutex<Option<Writer>>,
     collector: Mutex<Option<Collector>>,
-    progress: Mutex<Option<ProgressCallback>>,
-    error_callback: Mutex<Option<Box<dyn Fn(String) + 'static + Sync + Send>>>,
+    /// Progress and error callbacks
+    callbacks: Mutex<CCallbacks>,
     /// Bool set to true when the thread has been set up,
     /// prevents re-setting of the thread after `finish()`
     write_thread: Mutex<(bool, Option<thread::JoinHandle<GifskiError>>)>,
@@ -125,8 +127,10 @@ pub unsafe extern "C" fn gifski_new(settings: *const GifskiSettings) -> *const G
             writer: Mutex::new(Some(writer)),
             write_thread: Mutex::new((false, None)),
             collector: Mutex::new(Some(collector)),
-            progress: Mutex::new(None),
-            error_callback: Mutex::new(None),
+            callbacks: Mutex::new(CCallbacks {
+                progress: None,
+                error: None,
+            }),
         }))
         .cast::<GifskiHandle>()
     } else {
@@ -200,11 +204,11 @@ pub unsafe extern "C" fn gifski_add_fixed_color(handle: *mut GifskiHandle, col_r
     }
 }
 
-/// Adds a frame to the animation. This function is asynchronous.
+/// Insert a new frame at the position of `frame_number`.
 ///
 /// File path must be valid UTF-8.
 ///
-/// `frame_number` orders frames (consecutive numbers starting from 0).
+/// Frame numbers start and 0, and must be consecutive (without gaps).
 /// You can add frames in any order, and they will be sorted by their `frame_number`.
 ///
 /// Presentation timestamp (PTS) is time in seconds, since start of the file, when this frame is to be displayed.
@@ -213,7 +217,7 @@ pub unsafe extern "C" fn gifski_add_fixed_color(handle: *mut GifskiHandle, col_r
 ///
 /// The first frame should have PTS=0. If the first frame has PTS > 0, it'll be used as a delay after the last frame.
 ///
-/// This function may block and wait until the frame is processed. Make sure to call `gifski_set_write_callback` or `gifski_set_file_output` first to avoid a deadlock.
+/// This function may block and wait until encoding can start. Make sure to call `gifski_set_write_callback` or `gifski_set_file_output` first to avoid a deadlock.
 ///
 /// Returns 0 (`GIFSKI_OK`) on success, and non-0 `GIFSKI_*` constant on error.
 #[no_mangle]
@@ -237,9 +241,20 @@ pub unsafe extern "C" fn gifski_add_frame_png_file(handle: *const GifskiHandle,
     }
 }
 
-/// Pixels is an array width×height×4 bytes large. The array is copied, so you can free/reuse it immediately.
+/// Insert a new frame at the position of `frame_number`.
 ///
-/// Presentation timestamp (PTS) is time in seconds, since start of the file (at 0), when this frame is to be displayed.
+/// `pixels` is an array width×height×4 bytes large.
+/// The array is copied, so you can free/reuse it immediately after this function returns.
+///
+/// Frame numbers start and 0, and must be consecutive (without gaps). However, you can add frames in any order.
+/// Order of calls to `gifski_add_frame_rgba` doesn't affect the animation.
+/// When frames are added out of order, the encoder will buffer them in RAM until they can be processed.
+/// Frames are always encoded in the order sorted by their `frame_number`.
+///
+/// Make sure to start frame numbers at 0 and don't leave any gaps, because otherwise the encoder may wait
+/// forever for a frame number that never arrives. It's an error to add the same `frame_number` more than once.
+///
+/// Presentation timestamp (PTS) is time in seconds, since start of the file, when this frame is to be displayed.
 /// For a 20fps video it could be `frame_number/20.0`.
 /// Frames with duplicate or out-of-order PTS will be skipped.
 ///
@@ -247,7 +262,7 @@ pub unsafe extern "C" fn gifski_add_frame_png_file(handle: *const GifskiHandle,
 ///
 /// Colors are in sRGB, uncorrelated RGBA, with alpha byte last.
 ///
-/// This function may block and wait until the frame is processed. Make sure to call `gifski_set_write_callback` or `gifski_set_file_output` first to avoid a deadlock.
+/// This function may block and wait until encoding can start. Make sure to call `gifski_set_write_callback` or `gifski_set_file_output` first to avoid a deadlock.
 ///
 /// Returns 0 (`GIFSKI_OK`) on success, and non-0 `GIFSKI_*` constant on error.
 #[no_mangle]
@@ -307,6 +322,10 @@ fn add_frame_rgba(handle: *const GifskiHandle, frame_number: u32, frame: ImgVec<
 /// Colors are in sRGB, uncorrelated ARGB, with alpha byte first.
 ///
 /// `gifski_add_frame_rgba` is preferred over this function.
+///
+/// The `pixels` array is copied, so you can free/reuse it immediately after this function returns.
+///
+/// This function may block and wait until encoding can start. Make sure to call `gifski_set_write_callback` first to avoid a deadlock.
 #[no_mangle]
 pub unsafe extern "C" fn gifski_add_frame_argb(handle: *const GifskiHandle, frame_number: u32, width: u32, bytes_per_row: u32, height: u32, pixels: *const ARGB8, presentation_timestamp: f64) -> GifskiError {
     let (pixels, stride) = match pixels_slice(pixels, width, height, bytes_per_row) {
@@ -329,10 +348,12 @@ pub unsafe extern "C" fn gifski_add_frame_argb(handle: *const GifskiHandle, fram
 /// Bytes per row must be multiple of 3 and greater or equal width×3.
 ///
 /// Colors are in sRGB, red byte first.
-
-/// This function may block and wait until the frame is processed. Make sure to call `gifski_set_write_callback` first to avoid a deadlock.
 ///
 /// `gifski_add_frame_rgba` is preferred over this function.
+///
+/// The `pixels` array is copied, so you can free/reuse it immediately after this function returns.
+///
+/// This function may block and wait until encoding can start. Make sure to call `gifski_set_write_callback` first to avoid a deadlock.
 #[no_mangle]
 pub unsafe extern "C" fn gifski_add_frame_rgb(handle: *const GifskiHandle, frame_number: u32, width: u32, bytes_per_row: u32, height: u32, pixels: *const RGB8, presentation_timestamp: f64) -> GifskiError {
     let (pixels, stride) = match pixels_slice(pixels, width, height, bytes_per_row) {
@@ -360,18 +381,12 @@ pub unsafe extern "C" fn gifski_add_frame_rgb(handle: *const GifskiHandle, frame
 /// This function must be called before `gifski_set_file_output()` to take effect.
 #[no_mangle]
 pub unsafe extern "C" fn gifski_set_progress_callback(handle: *const GifskiHandle, cb: unsafe extern "C" fn(*mut c_void) -> c_int, user_data: *mut c_void) -> GifskiError {
-    let Some(g) = borrow(handle) else { return GifskiError::NULL_ARG };
-
-    if g.write_thread.lock().map_or(true, |t| t.0) {
-        g.print_error("tried to set progress callback after writing has already started".into());
-        return GifskiError::INVALID_STATE;
-    }
-    match g.progress.lock() {
-        Ok(mut progress) => {
-            *progress = Some(ProgressCallback::new(cb, user_data));
+    match set_callbacks(borrow(handle)) {
+        Ok(mut callbacks) => {
+            callbacks.progress = Some(ProgressCallback::new(cb, user_data));
             GifskiError::OK
         },
-        Err(_) => GifskiError::THREAD_LOST,
+        Err(e) => e,
     }
 }
 
@@ -390,27 +405,28 @@ pub unsafe extern "C" fn gifski_set_progress_callback(handle: *const GifskiHandl
 /// This function must be called before `gifski_set_file_output()` to take effect.
 #[no_mangle]
 pub unsafe extern "C" fn gifski_set_error_message_callback(handle: *const GifskiHandle, cb: unsafe extern "C" fn(*const c_char, *mut c_void), user_data: *mut c_void) -> GifskiError {
-    let Some(g) = borrow(handle) else { return GifskiError::NULL_ARG };
-
-    let user_data = SendableUserData(user_data);
-    match g.error_callback.lock() {
-        Ok(mut error_callback) => {
-            *error_callback = Some(Box::new(move |mut s: String| {
-                s.reserve_exact(1);
-                s.push('\0');
-                let cstring = CString::from_vec_with_nul(s.into_bytes()).unwrap_or_default();
-                unsafe { cb(cstring.as_ptr(), user_data.clone().0) } // the clone is a no-op, only to force closure to own it
-            }));
+    match set_callbacks(borrow(handle)) {
+        Ok(mut callbacks) => {
+            callbacks.error = Some(ErrorCallback {
+                callback: cb,
+                user_data,
+            });
             GifskiError::OK
         },
-        Err(_) => GifskiError::THREAD_LOST,
+        Err(e) => e,
     }
 }
 
-#[derive(Clone)]
-struct SendableUserData(*mut c_void);
-unsafe impl Send for SendableUserData {}
-unsafe impl Sync for SendableUserData {}
+fn set_callbacks(g: Option<&GifskiHandleInternal>) -> Result<MutexGuard<'_, CCallbacks>, GifskiError> {
+    let g = g.ok_or(GifskiError::NULL_ARG)?;
+
+    if g.write_thread.lock().map_or(true, |t| t.0) {
+        g.print_error("tried to set a callback after writing has already started".into());
+        return Err(GifskiError::INVALID_STATE);
+    }
+
+    g.callbacks.lock().map_err(|_| GifskiError::THREAD_LOST)
+}
 
 /// Start writing to the `destination`. This has to be called before any frames are added.
 ///
@@ -505,11 +521,10 @@ fn gifski_write_thread_start<W: 'static +  Write + Send>(g: &GifskiHandleInterna
         return Err(GifskiError::INVALID_STATE);
     }
     let writer = g.writer.lock().map_err(|_| GifskiError::THREAD_LOST)?.take();
-    let mut user_progress = g.progress.lock().map_err(|_| GifskiError::THREAD_LOST)?.take();
+    let mut user_progress = g.callbacks.lock().map_err(|_| GifskiError::THREAD_LOST)?.clone();
     let handle = thread::Builder::new().name("c-write".into()).spawn(move || {
         if let Some(writer) = writer {
-            let progress = user_progress.as_mut().map(|m| m as &mut dyn ProgressReporter);
-            match writer.write(file, progress.unwrap_or(&mut NoProgress {})).into() {
+            match writer.write(file, &mut user_progress).into() {
                 res @ (GifskiError::OK | GifskiError::ALREADY_EXISTS) => res,
                 err => {
                     if let Some(path) = path {
@@ -578,9 +593,10 @@ pub unsafe extern "C" fn gifski_finish(g: *const GifskiHandle) -> GifskiError {
 }
 
 impl GifskiHandleInternal {
+    #[cold]
     fn print_error(&self, mut err: String) {
-        if let Ok(Some(cb)) = self.error_callback.lock().as_deref() {
-            cb(err);
+        if let Ok(reporter) = self.callbacks.lock().as_deref_mut() {
+            reporter.error(err);
         } else {
             err.reserve_exact(1);
             err.push('\n');
@@ -588,6 +604,7 @@ impl GifskiHandleInternal {
         }
     }
 
+    #[cold]
     fn print_panic(&self, e: Box<dyn std::any::Any + Send>) {
         let msg = e.downcast_ref::<String>().map(|s| s.as_str())
             .or_else(|| e.downcast_ref::<&str>().copied()).unwrap_or("unknown panic");
@@ -625,7 +642,7 @@ fn c_cb() {
         assert_eq!(GifskiError::OK, gifski_set_write_callback(g, Some(cb), ptr::addr_of_mut!(write_called).cast()));
         assert_eq!(GifskiError::INVALID_STATE, gifski_set_progress_callback(g, pcb, ptr::addr_of_mut!(progress_called).cast()));
         assert_eq!(GifskiError::OK, gifski_add_frame_rgb(g, 0, 1, 3, 1, &RGB::new(0,0,0), 3.));
-        assert_eq!(GifskiError::OK, gifski_add_frame_rgb(g, 0, 1, 3, 1, &RGB::new(0,0,0), 10.));
+        assert_eq!(GifskiError::OK, gifski_add_frame_rgb(g, 1, 1, 3, 1, &RGB::new(0,0,0), 10.));
         assert_eq!(GifskiError::OK, gifski_finish(g));
     }
     assert!(write_called);
@@ -655,7 +672,7 @@ fn progress_abort() {
         assert_eq!(GifskiError::OK, gifski_set_progress_callback(g, pcb, ptr::null_mut()));
         assert_eq!(GifskiError::OK, gifski_set_write_callback(g, Some(cb), ptr::null_mut()));
         assert_eq!(GifskiError::OK, gifski_add_frame_rgb(g, 0, 1, 3, 1, &RGB::new(0, 0, 0), 3.));
-        assert_eq!(GifskiError::OK, gifski_add_frame_rgb(g, 0, 1, 3, 1, &RGB::new(0, 0, 0), 10.));
+        assert_eq!(GifskiError::OK, gifski_add_frame_rgb(g, 1, 1, 3, 1, &RGB::new(0, 0, 0), 10.));
         assert_eq!(GifskiError::ABORTED, gifski_finish(g));
     }
 }
@@ -722,7 +739,7 @@ fn test_error_callback() {
         assert_eq!(GifskiError::OK, gifski_set_write_callback(g, Some(cb), 1 as _));
         assert_eq!(GifskiError::INVALID_STATE, gifski_set_write_callback(g, Some(cb), 1 as _));
         assert_eq!(GifskiError::INVALID_STATE, gifski_finish(g));
-        assert_eq!("gifski_set_file_output/gifski_set_write_callback has been called already", callback_msg.unwrap());
+        assert_eq!("gifski_set_file_output/gifski_set_write_callback has been called already", callback_msg.expect("set msg"));
     }
 }
 
